import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { CreateAdCommand } from './create-ad.command';
import { Inject } from '@nestjs/common';
import {
  AD_REPOSITORY,
  INPUT_DATETIME_TRANSFORMER,
} from '@modules/ad/ad.di-tokens';
import { AdEntity } from '@modules/ad/core/domain/ad.entity';
import { Waypoint } from '../../types/waypoint';
import { AdRepositoryPort } from '../../ports/ad.repository.port';
import { AdAlreadyExistsException } from '@modules/ad/core/domain/ad.errors';
import { AggregateID, ConflictException } from '@mobicoop/ddd-library';
import { ScheduleItem } from '../../types/schedule-item';
import { DateTimeTransformerPort } from '../../ports/datetime-transformer.port';

@CommandHandler(CreateAdCommand)
export class CreateAdService implements ICommandHandler {
  constructor(
    @Inject(AD_REPOSITORY)
    private readonly repository: AdRepositoryPort,
    @Inject(INPUT_DATETIME_TRANSFORMER)
    private readonly datetimeTransformer: DateTimeTransformerPort,
  ) {}

  async execute(command: CreateAdCommand): Promise<AggregateID> {
    const ad = AdEntity.create({
      userId: command.userId,
      driver: command.driver,
      passenger: command.passenger,
      frequency: command.frequency,
      fromDate: this.datetimeTransformer.fromDate(
        {
          date: command.fromDate,
          time: command.schedule[0].time,
          coordinates: {
            lon: command.waypoints[0].lon,
            lat: command.waypoints[0].lat,
          },
        },
        command.frequency,
      ),
      toDate: this.datetimeTransformer.toDate(
        command.toDate,
        {
          date: command.fromDate,
          time: command.schedule[0].time,
          coordinates: {
            lon: command.waypoints[0].lon,
            lat: command.waypoints[0].lat,
          },
        },
        command.frequency,
      ),
      schedule: command.schedule.map((scheduleItem: ScheduleItem) => ({
        day: this.datetimeTransformer.day(
          scheduleItem.day,
          {
            date: command.fromDate,
            time: scheduleItem.time,
            coordinates: {
              lon: command.waypoints[0].lon,
              lat: command.waypoints[0].lat,
            },
          },
          command.frequency,
        ),
        time: this.datetimeTransformer.time(
          {
            date: command.fromDate,
            time: scheduleItem.time,
            coordinates: {
              lon: command.waypoints[0].lon,
              lat: command.waypoints[0].lat,
            },
          },
          command.frequency,
        ),
        margin: scheduleItem.margin,
      })),
      seatsProposed: command.seatsProposed ?? 0,
      seatsRequested: command.seatsRequested ?? 0,
      strict: command.strict,
      waypoints: command.waypoints.map((waypoint: Waypoint) => ({
        position: waypoint.position,
        address: {
          name: waypoint.name,
          houseNumber: waypoint.houseNumber,
          street: waypoint.street,
          postalCode: waypoint.postalCode,
          locality: waypoint.locality,
          country: waypoint.country,
          coordinates: {
            lon: waypoint.lon,
            lat: waypoint.lat,
          },
        },
      })),
      comment: command.comment,
    });

    try {
      await this.repository.insert(ad);
      return ad.id;
    } catch (error: any) {
      if (error instanceof ConflictException) {
        throw new AdAlreadyExistsException(error);
      }
      throw error;
    }
  }
}
