import { Inject, Injectable } from '@nestjs/common';
import {
  DateTimeTransformerPort,
  Frequency,
  GeoDateTime,
} from '../core/application/ports/datetime-transformer.port';
import { TimeConverterPort } from '../core/application/ports/time-converter.port';
import { TIMEZONE_FINDER, TIME_CONVERTER } from '../ad.di-tokens';
import { TimezoneFinderPort } from '../core/application/ports/timezone-finder.port';

@Injectable()
export class OutputDateTimeTransformer implements DateTimeTransformerPort {
  constructor(
    @Inject(TIMEZONE_FINDER)
    private readonly timezoneFinder: TimezoneFinderPort,
    @Inject(TIME_CONVERTER) private readonly timeConverter: TimeConverterPort,
  ) {}

  /**
   * Compute the fromDate : if an ad is punctual, the departure date
   * is converted from UTC to the local date with the time and timezone
   */
  fromDate = (geoFromDate: GeoDateTime, frequency: Frequency): string => {
    if (frequency === Frequency.RECURRENT) return geoFromDate.date;
    return this.timeConverter
      .utcStringDateTimeToLocalIsoString(
        geoFromDate.date,
        geoFromDate.time,
        this.timezoneFinder.timezones(
          geoFromDate.coordinates.lon,
          geoFromDate.coordinates.lat,
        )[0],
      )
      .split('T')[0];
  };

  /**
   * Get the toDate depending on frequency, time and timezone :
   * if the ad is punctual, the toDate is equal to the fromDate
   */
  toDate = (
    toDate: string,
    geoFromDate: GeoDateTime,
    frequency: Frequency,
  ): string => {
    if (frequency === Frequency.RECURRENT) return toDate;
    return this.fromDate(geoFromDate, frequency);
  };

  /**
   * Get the day for a schedule item :
   * - if the ad is punctual, the day is infered from fromDate
   * - if the ad is recurrent, the day is computed by converting the time from utc to local time
   */
  day = (
    day: number,
    geoFromDate: GeoDateTime,
    frequency: Frequency,
  ): number => {
    if (frequency === Frequency.RECURRENT)
      return this.recurrentDay(
        day,
        geoFromDate.time,
        this.timezoneFinder.timezones(
          geoFromDate.coordinates.lon,
          geoFromDate.coordinates.lat,
        )[0],
      );
    return new Date(this.fromDate(geoFromDate, frequency)).getUTCDay();
  };

  /**
   * Get the utc time
   */
  time = (geoFromDate: GeoDateTime, frequency: Frequency): string => {
    if (frequency === Frequency.RECURRENT)
      return this.timeConverter.utcStringTimeToLocalStringTime(
        geoFromDate.time,
        this.timezoneFinder.timezones(
          geoFromDate.coordinates.lon,
          geoFromDate.coordinates.lat,
        )[0],
      );
    return this.timeConverter
      .utcStringDateTimeToLocalIsoString(
        geoFromDate.date,
        geoFromDate.time,
        this.timezoneFinder.timezones(
          geoFromDate.coordinates.lon,
          geoFromDate.coordinates.lat,
        )[0],
      )
      .split('T')[1]
      .split(':', 2)
      .join(':');
  };

  /**
   * Get the day for a schedule item for a recurrent ad
   * The day may change when transforming from utc to local timezone
   */
  private recurrentDay = (
    day: number,
    time: string,
    timezone: string,
  ): number => {
    const unixEpochDay = 4; // 1970-01-01 is a thursday !
    const localBaseDay = this.timeConverter.localUnixEpochDayFromTime(
      time,
      timezone,
    );
    if (unixEpochDay == localBaseDay) return day;
    if (unixEpochDay > localBaseDay) return day > 0 ? day - 1 : 6;
    return day < 6 ? day + 1 : 0;
  };
}
