import { Test, TestingModule } from '@nestjs/testing';
import {
  AD_REPOSITORY,
  INPUT_DATETIME_TRANSFORMER,
} from '@modules/ad/ad.di-tokens';
import { WaypointDto } from '@modules/ad/interface/grpc-controllers/dtos/waypoint.dto';
import { CreateAdRequestDto } from '@modules/ad/interface/grpc-controllers/dtos/create-ad.request.dto';
import { AggregateID } from '@mobicoop/ddd-library';
import { AdEntity } from '@modules/ad/core/domain/ad.entity';
import { ConflictException } from '@mobicoop/ddd-library';
import { Frequency } from '@modules/ad/core/domain/ad.types';
import { CreateAdService } from '@modules/ad/core/application/commands/create-ad/create-ad.service';
import { CreateAdCommand } from '@modules/ad/core/application/commands/create-ad/create-ad.command';
import { AdAlreadyExistsException } from '@modules/ad/core/domain/ad.errors';
import { DateTimeTransformerPort } from '@modules/ad/core/application/ports/datetime-transformer.port';

const originWaypoint: WaypointDto = {
  position: 0,
  lat: 48.689445,
  lon: 6.17651,
  houseNumber: '5',
  street: 'Avenue Foch',
  locality: 'Nancy',
  postalCode: '54000',
  country: 'France',
};
const destinationWaypoint: WaypointDto = {
  position: 1,
  lat: 48.8566,
  lon: 2.3522,
  locality: 'Paris',
  postalCode: '75000',
  country: 'France',
};
const punctualCreateAdRequest: CreateAdRequestDto = {
  userId: '4eb6a6af-ecfd-41c3-9118-473a507014d4',
  fromDate: '2023-12-21',
  toDate: '2023-12-21',
  schedule: [
    {
      time: '08:15',
      margin: 900,
      day: 4,
    },
  ],
  driver: true,
  passenger: true,
  seatsRequested: 1,
  seatsProposed: 3,
  strict: false,
  frequency: Frequency.PUNCTUAL,
  waypoints: [originWaypoint, destinationWaypoint],
};

const mockAdRepository = {
  insert: jest
    .fn()
    .mockImplementationOnce(() => ({}))
    .mockImplementationOnce(() => {
      throw new Error();
    })
    .mockImplementationOnce(() => {
      throw new ConflictException('already exists');
    }),
};

const mockInputDateTimeTransformer: DateTimeTransformerPort = {
  fromDate: jest.fn(),
  toDate: jest.fn(),
  day: jest.fn(),
  time: jest.fn(),
};

describe('create-ad.service', () => {
  let createAdService: CreateAdService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: AD_REPOSITORY,
          useValue: mockAdRepository,
        },
        {
          provide: INPUT_DATETIME_TRANSFORMER,
          useValue: mockInputDateTimeTransformer,
        },
        CreateAdService,
      ],
    }).compile();

    createAdService = module.get<CreateAdService>(CreateAdService);
  });

  it('should be defined', () => {
    expect(createAdService).toBeDefined();
  });

  describe('execution', () => {
    const createAdCommand = new CreateAdCommand(punctualCreateAdRequest);
    it('should create a new punctual ad', async () => {
      AdEntity.create = jest.fn().mockReturnValue({
        id: '047a6ecf-23d4-4d3e-877c-3225d560a8da',
      });
      const result: AggregateID =
        await createAdService.execute(createAdCommand);
      expect(result).toBe('047a6ecf-23d4-4d3e-877c-3225d560a8da');
    });
    it('should throw an error if something bad happens', async () => {
      AdEntity.create = jest.fn().mockReturnValue({
        id: '047a6ecf-23d4-4d3e-877c-3225d560a8da',
      });
      await expect(
        createAdService.execute(createAdCommand),
      ).rejects.toBeInstanceOf(Error);
    });
    it('should throw an exception if Ad already exists', async () => {
      AdEntity.create = jest.fn().mockReturnValue({
        id: '047a6ecf-23d4-4d3e-877c-3225d560a8da',
      });
      await expect(
        createAdService.execute(createAdCommand),
      ).rejects.toBeInstanceOf(AdAlreadyExistsException);
    });
  });
});
