import { RpcExceptionCode } from '@mobicoop/ddd-library';
import { AdMapper } from '@modules/ad/ad.mapper';
import { Frequency } from '@modules/ad/core/domain/ad.types';
import { FindAdsByUserIdGrpcController } from '@modules/ad/interface/grpc-controllers/find-ads-by-user-id.grpc.controller';
import { QueryBus } from '@nestjs/cqrs';
import { RpcException } from '@nestjs/microservices';
import { Test, TestingModule } from '@nestjs/testing';

const mockQueryBus = {
  execute: jest
    .fn()
    .mockImplementationOnce(() => [
      '200d61a8-d878-4378-a609-c19ea71633d2',
      '200d61a8-d878-4378-a609-c19ea71633d3',
      '200d61a8-d878-4378-a609-c19ea71633d4',
    ])
    .mockImplementationOnce(() => {
      throw new Error();
    }),
};

const mockAdMapper = {
  toResponse: jest.fn().mockImplementationOnce(() => ({
    userId: '8cc90d1a-4a59-4289-a7d8-078f9db7857f',
    driver: true,
    passenger: true,
    frequency: Frequency.PUNCTUAL,
    fromDate: '2023-06-27',
    toDate: '2023-06-27',
    schedule: {
      tue: '07:15',
    },
    marginDurations: {
      mon: 900,
      tue: 900,
      wed: 900,
      thu: 900,
      fri: 900,
      sat: 900,
      sun: 900,
    },
    seatsProposed: 3,
    seatsRequested: 1,
    waypoints: [
      {
        position: 0,
        lat: 48.689445,
        lon: 6.17651,
        houseNumber: '5',
        street: 'Avenue Foch',
        locality: 'Nancy',
        postalCode: '54000',
        country: 'France',
      },
      {
        position: 1,
        lat: 48.8566,
        lon: 2.3522,
        locality: 'Paris',
        postalCode: '75000',
        country: 'France',
      },
    ],
  })),
};

describe('Find Ads By User Id Grpc Controller', () => {
  let findAdsByUserIdGrpcController: FindAdsByUserIdGrpcController;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: QueryBus,
          useValue: mockQueryBus,
        },
        {
          provide: AdMapper,
          useValue: mockAdMapper,
        },
        FindAdsByUserIdGrpcController,
      ],
    }).compile();

    findAdsByUserIdGrpcController = module.get<FindAdsByUserIdGrpcController>(
      FindAdsByUserIdGrpcController,
    );
  });

  afterEach(async () => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(findAdsByUserIdGrpcController).toBeDefined();
  });

  it('should return ads', async () => {
    jest.spyOn(mockQueryBus, 'execute');
    jest.spyOn(mockAdMapper, 'toResponse');
    const response = await findAdsByUserIdGrpcController.findAllByUserId({
      id: '8cc90d1a-4a59-4289-a7d8-078f9db7857f',
    });
    expect(response.ads).toHaveLength(3);
    expect(mockQueryBus.execute).toHaveBeenCalledTimes(1);
    expect(mockAdMapper.toResponse).toHaveBeenCalledTimes(3);
  });

  it('should throw a generic RpcException', async () => {
    jest.spyOn(mockQueryBus, 'execute');
    jest.spyOn(mockAdMapper, 'toResponse');
    expect.assertions(4);
    try {
      await findAdsByUserIdGrpcController.findAllByUserId({
        id: '8cc90d1a-4a59-4289-a7d8-078f9db7857f',
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.UNKNOWN);
    }
    expect(mockQueryBus.execute).toHaveBeenCalledTimes(1);
    expect(mockAdMapper.toResponse).toHaveBeenCalledTimes(0);
  });
});
